@@ -37,15 +37,12 @@ private Q_SLOTS:
 void ShowingDesktopTest::initTestCase()
 {
     qRegisterMetaType<KWin::Window *>();
-    QSignalSpy applicationStartedSpy(kwinApp(), &Application::started);
     QVERIFY(waylandServer()->init(s_socketName));
+    kwinApp()->start();
     Test::setOutputConfig({
         QRect(0, 0, 1280, 1024),
         QRect(1280, 0, 1280, 1024),
     });
-
-    kwinApp()->start();
-    QVERIFY(applicationStartedSpy.wait());
 }
 
 void ShowingDesktopTest::init()
