@@ -13,7 +13,7 @@
 class OnScreenNotificationTest : public QObject
 {
     Q_OBJECT
-private slots:
+private Q_SLOTS:
 
     void show();
     void timeout();
