@@ -11,15 +11,11 @@
 #include <map>
 #include <memory>
 
-namespace KWaylandServer
-{
-class FakeInputInterface;
-class FakeInputDevice;
-}
-
 namespace KWin
 {
 
+class Display;
+class FakeInputBackendPrivate;
 class FakeInputDevice;
 
 class FakeInputBackend : public InputBackend
@@ -27,14 +23,13 @@ class FakeInputBackend : public InputBackend
     Q_OBJECT
 
 public:
-    explicit FakeInputBackend();
+    explicit FakeInputBackend(Display *display);
     ~FakeInputBackend();
 
     void initialize() override;
 
 private:
-    std::unique_ptr<KWaylandServer::FakeInputInterface> m_interface;
-    std::map<KWaylandServer::FakeInputDevice *, std::unique_ptr<FakeInputDevice>> m_devices;
+    std::unique_ptr<FakeInputBackendPrivate> d;
 };
 
 } // namespace KWin
