@@ -158,11 +158,11 @@ uint LogindSession::terminal() const
     return m_terminal;
 }
 
-int LogindSession::openRestricted(const QString &fileName)
+std::expected<int, Session::Error> LogindSession::openRestricted(const QString &fileName)
 {
     struct stat st;
     if (stat(fileName.toUtf8(), &st) < 0) {
-        return -1;
+        return std::unexpected(errorFromErrno());
     }
 
     QDBusMessage message = QDBusMessage::createMethodCall(s_serviceName, m_sessionPath,
@@ -175,16 +175,24 @@ int LogindSession::openRestricted(const QString &fileName)
     if (reply.type() == QDBusMessage::ErrorMessage) {
         qCWarning(KWIN_CORE, "Failed to open %s device (%s)",
                   qPrintable(fileName), qPrintable(reply.errorMessage()));
-        return -1;
+        if (reply.errorName() == "System.Error.EBUSY") {
+            return std::unexpected(Error::EBusy);
+        } else {
+            return std::unexpected(Error::Other);
+        }
     }
 
     const QDBusUnixFileDescriptor descriptor = reply.arguments().constFirst().value<QDBusUnixFileDescriptor>();
     if (!descriptor.isValid()) {
-        qCWarning(KWIN_CORE, "File descriptor for %s from logind is invalid", qPrintable(fileName));
-        return -1;
+        return std::unexpected(Error::Other);
     }
 
-    return fcntl(descriptor.fileDescriptor(), F_DUPFD_CLOEXEC, 0);
+    const int ret = fcntl(descriptor.fileDescriptor(), F_DUPFD_CLOEXEC, 0);
+    if (ret == -1) {
+        return std::unexpected(errorFromErrno());
+    } else {
+        return ret;
+    }
 }
 
 void LogindSession::closeRestricted(int fileDescriptor)
@@ -216,6 +224,22 @@ void LogindSession::switchTo(uint terminal)
     QDBusConnection::systemBus().asyncCall(message);
 }
 
+FileDescriptor LogindSession::delaySleep(const QString &reason)
+{
+    QDBusMessage message = QDBusMessage::createMethodCall(s_serviceName, m_seatPath,
+                                                          s_seatInterface,
+                                                          QStringLiteral("Inhibit"));
+    message.setArguments({QStringLiteral("sleep"), QStringLiteral("compositor"), reason, QStringLiteral("delay")});
+
+    const QDBusMessage reply = QDBusConnection::systemBus().call(message);
+    if (reply.type() == QDBusMessage::ErrorMessage) {
+        qCWarning(KWIN_CORE, "Failed to delay sleep: %s", qPrintable(reply.errorMessage()));
+        return FileDescriptor{};
+    }
+    const QDBusUnixFileDescriptor descriptor = reply.arguments().constFirst().value<QDBusUnixFileDescriptor>();
+    return FileDescriptor{descriptor.fileDescriptor()};
+}
+
 LogindSession::LogindSession(const QString &sessionPath)
     : m_sessionPath(sessionPath)
 {
@@ -342,7 +366,9 @@ void LogindSession::handlePropertiesChanged(const QString &interfaceName, const
 
 void LogindSession::handlePrepareForSleep(bool sleep)
 {
-    if (!sleep) {
+    if (sleep) {
+        Q_EMIT aboutToSleep();
+    } else {
         Q_EMIT awoke();
     }
 }
