@@ -231,12 +231,15 @@ int main(int argc, char *argv[])
     hostname = hostname.toHtmlEscaped();
     QString pidString = QString::number(pid); // format pid ourself as it does not make sense to format an ID according to locale settings
 
-    QString question = (caption == appname) ? xi18nc("@info", "<para><application>%1</application> is not responding. Do you want to terminate this application?</para>",
-                                                     appname)
-                                            : xi18nc("@info \"window title\" of application name is not responding.", "<para>\"%1\" of <application>%2</application> is not responding. Do you want to terminate this application?</para>",
-                                                     caption, appname);
-    question += xi18nc("@info",
-                       "<para><emphasis strong='true'>Terminating this application will close all of its windows. Any unsaved data will be lost.</emphasis></para>");
+    const QString question = (caption == appname)
+        ? xi18nc("@info",
+                 "<para><application>%1</application> is not responding. Do you want to terminate this application?</para>"
+                 "<para><emphasis strong='true'>Terminating this application will close all of its windows. Any unsaved data will be lost.</emphasis></para>",
+                 appname)
+        : xi18nc("@info \"window title\" of application name is not responding.",
+                 "<para>\"%1\" of <application>%2</application> is not responding. Do you want to terminate this application?</para>"
+                 "<para><emphasis strong='true'>Terminating this application will close all of its windows. Any unsaved data will be lost.</emphasis></para>",
+                 caption, appname);
 
     KGuiItem continueButton = KGuiItem(i18nc("@action:button Terminate app", "&Terminate %1", appname), QStringLiteral("application-exit"));
     KGuiItem cancelButton = KGuiItem(i18nc("@action:button Wait for frozen app to maybe respond again", "&Wait Longer"), QStringLiteral("chronometer"));
