@@ -10,7 +10,6 @@
 
 #include "input.h"
 
-#include <QInputEvent>
 #include <chrono>
 
 namespace KWin
@@ -60,6 +59,88 @@ struct PointerAxisEvent
     std::chrono::microseconds timestamp;
 };
 
+struct PointerSwipeGestureBeginEvent
+{
+    int fingerCount;
+    std::chrono::microseconds time;
+};
+
+struct PointerSwipeGestureUpdateEvent
+{
+    QPointF delta;
+    std::chrono::microseconds time;
+};
+
+struct PointerSwipeGestureEndEvent
+{
+    std::chrono::microseconds time;
+};
+
+struct PointerSwipeGestureCancelEvent
+{
+    std::chrono::microseconds time;
+};
+
+struct PointerPinchGestureBeginEvent
+{
+    int fingerCount;
+    std::chrono::microseconds time;
+};
+
+struct PointerPinchGestureUpdateEvent
+{
+    qreal scale;
+    qreal angleDelta;
+    QPointF delta;
+    std::chrono::microseconds time;
+};
+
+struct PointerPinchGestureEndEvent
+{
+    std::chrono::microseconds time;
+};
+
+struct PointerPinchGestureCancelEvent
+{
+    std::chrono::microseconds time;
+};
+
+struct PointerHoldGestureBeginEvent
+{
+    int fingerCount;
+    std::chrono::microseconds time;
+};
+
+struct PointerHoldGestureEndEvent
+{
+    std::chrono::microseconds time;
+};
+
+struct PointerHoldGestureCancelEvent
+{
+    std::chrono::microseconds time;
+};
+
+struct TouchDownEvent
+{
+    qint32 id;
+    QPointF pos;
+    std::chrono::microseconds time;
+};
+
+struct TouchMotionEvent
+{
+    qint32 id;
+    QPointF pos;
+    std::chrono::microseconds time;
+};
+
+struct TouchUpEvent
+{
+    qint32 id;
+    std::chrono::microseconds time;
+};
+
 struct KeyboardKeyEvent
 {
     InputDevice *device;
@@ -71,6 +152,7 @@ struct KeyboardKeyEvent
     Qt::KeyboardModifiers modifiers;
     Qt::KeyboardModifiers modifiersRelevantForGlobalShortcuts;
     std::chrono::microseconds timestamp;
+    uint32_t serial;
 };
 
 struct SwitchEvent
@@ -80,27 +162,62 @@ struct SwitchEvent
     std::chrono::microseconds timestamp;
 };
 
-class TabletEvent : public QTabletEvent
+struct TabletToolProximityEvent
 {
 public:
-    TabletEvent(Type t, const QPointingDevice *dev, const QPointF &pos, const QPointF &globalPos,
-                qreal pressure, float xTilt, float yTilt,
-                float tangentialPressure, qreal rotation, float z,
-                Qt::KeyboardModifiers keyState, Qt::MouseButton button, Qt::MouseButtons buttons, InputDeviceTabletTool *tool, InputDevice *device);
+    enum Type {
+        EnterProximity,
+        LeaveProximity,
+    };
 
-    InputDevice *device() const
-    {
-        return m_device;
-    }
+    Type type;
+    InputDevice *device;
+    qreal rotation;
+    QPointF position;
+    qreal sliderPosition;
+    qreal xTilt;
+    qreal yTilt;
+    qreal distance;
+    std::chrono::microseconds timestamp;
+    InputDeviceTabletTool *tool;
+};
 
-    InputDeviceTabletTool *tool() const
-    {
-        return m_tool;
-    }
+struct TabletToolTipEvent
+{
+public:
+    enum Type {
+        Press,
+        Release,
+    };
 
-private:
-    InputDeviceTabletTool *m_tool;
-    InputDevice *m_device;
+    Type type;
+    InputDevice *device;
+    qreal rotation;
+    QPointF position;
+    Qt::MouseButtons buttons;
+    qreal pressure;
+    qreal sliderPosition;
+    qreal xTilt;
+    qreal yTilt;
+    qreal distance;
+    std::chrono::microseconds timestamp;
+    InputDeviceTabletTool *tool;
+};
+
+struct TabletToolAxisEvent
+{
+public:
+    InputDevice *device;
+    qreal rotation;
+    QPointF position;
+    Qt::MouseButtons buttons;
+    qreal pressure;
+    qreal sliderPosition;
+    qreal xTilt;
+    qreal yTilt;
+    qreal distance;
+    std::chrono::microseconds timestamp;
+    InputDeviceTabletTool *tool;
 };
 
 struct TabletToolButtonEvent
@@ -117,6 +234,9 @@ struct TabletPadButtonEvent
     InputDevice *device;
     uint button;
     bool pressed;
+    quint32 group;
+    quint32 mode;
+    bool isModeSwitch;
     std::chrono::microseconds time;
 };
 
@@ -126,6 +246,8 @@ struct TabletPadStripEvent
     int number;
     int position;
     bool isFinger;
+    quint32 group;
+    quint32 mode;
     std::chrono::microseconds time;
 };
 
@@ -135,6 +257,17 @@ struct TabletPadRingEvent
     int number;
     int position;
     bool isFinger;
+    quint32 group;
+    quint32 mode;
+    std::chrono::microseconds time;
+};
+
+struct TabletPadDialEvent
+{
+    InputDevice *device;
+    int number;
+    double delta;
+    quint32 group;
     std::chrono::microseconds time;
 };
 
