@@ -42,7 +42,6 @@ public:
 
     enum EffectComboRole {
         LayoutPath = Qt::UserRole + 1,
-        AddonEffect, // i.e not builtin effects
     };
 
     explicit KWinTabBoxConfigForm(TabboxType type, TabBox::TabBoxSettings *config, TabBox::ShortcutSettings *shortcutsConfig, QWidget *parent = nullptr);
@@ -60,10 +59,9 @@ public:
 
 Q_SIGNALS:
     void configChanged();
-    void effectConfigButtonClicked();
+    void effectPreviewClicked();
 
 private Q_SLOTS:
-    void tabBoxToggled(bool on);
     void onFilterScreen();
     void onFilterDesktop();
     void onFilterActivites();
@@ -73,7 +71,6 @@ private Q_SLOTS:
     void onShowDesktopMode();
     void onSwitchingMode();
     void onEffectCombo();
-    void onShortcutChanged(const QKeySequence &seq);
     void updateDefaultIndicators();
 
 private:
@@ -108,7 +105,6 @@ private:
     TabBox::ShortcutSettings *m_shortcuts = nullptr;
     bool m_showDefaultIndicator = false;
 
-    bool m_isHighlightWindowsEnabled = true;
     Ui::KWinTabBoxConfigForm *ui;
 };
 
