@@ -19,7 +19,6 @@ namespace TabBox
 {
 class TabBoxSettings;
 class SwitchEffectSettings;
-class PluginsSettings;
 class ShortcutSettings;
 
 class KWinTabboxData : public KCModuleData
@@ -31,13 +30,11 @@ public:
 
     TabBoxSettings *tabBoxConfig() const;
     TabBoxSettings *tabBoxAlternativeConfig() const;
-    PluginsSettings *pluginsConfig() const;
     ShortcutSettings *shortcutConfig() const;
 
 private:
     TabBoxSettings *m_tabBoxConfig;
     TabBoxSettings *m_tabBoxAlternativeConfig;
-    PluginsSettings *m_pluginsConfig;
     ShortcutSettings *m_shortcutConfig;
 };
 
