@@ -39,10 +39,8 @@ struct Trigger
 {
     QString device;
     uint button;
-    bool operator==(const Trigger &o) const
-    {
-        return button == o.button && device == o.device;
-    }
+    quint32 mode;
+    bool operator==(const Trigger &o) const = default;
 };
 
 class ButtonRebindsFilter : public KWin::Plugin, public KWin::InputEventFilter
@@ -53,9 +51,18 @@ public:
         Pointer,
         TabletPad,
         TabletToolButtonType,
+        TabletDial,
+        TabletRing,
         LastType
     };
     Q_ENUM(TriggerType)
+    using SingleKeybind = QKeySequence;
+    struct AxisKeybind
+    {
+        QKeySequence up;
+        QKeySequence down;
+        int threshold = 120; // multiples of 120, or how many "twists" of the dial or ring is needed to emit this event
+    };
     struct TabletToolButton
     {
         quint32 button;
@@ -68,6 +75,9 @@ public:
     struct DisabledButton
     {
     };
+    struct ScrollWheel
+    {
+    };
 
     explicit ButtonRebindsFilter();
     ~ButtonRebindsFilter() override;
@@ -77,21 +87,25 @@ public:
     bool tabletToolTipEvent(KWin::TabletToolTipEvent *event) override;
     bool tabletPadButtonEvent(KWin::TabletPadButtonEvent *event) override;
     bool tabletToolButtonEvent(KWin::TabletToolButtonEvent *event) override;
+    bool tabletPadDialEvent(KWin::TabletPadDialEvent *event) override;
+    bool tabletPadRingEvent(KWin::TabletPadRingEvent *event) override;
 
 private:
     void loadConfig(const KConfigGroup &group);
     void insert(TriggerType type, const Trigger &trigger, const QStringList &action);
-    bool send(TriggerType type, const Trigger &trigger, bool pressed, std::chrono::microseconds timestamp);
+    bool send(TriggerType type, const Trigger &trigger, bool pressed, double delta, std::chrono::microseconds timestamp);
     bool sendKeySequence(const QKeySequence &sequence, bool pressed, std::chrono::microseconds time);
     bool sendKeyModifiers(const Qt::KeyboardModifiers &modifiers, bool pressed, std::chrono::microseconds time);
     bool sendMouseButton(quint32 button, bool pressed, std::chrono::microseconds time);
     bool sendMousePosition(QPointF position, std::chrono::microseconds time);
     bool sendMouseFrame();
     bool sendTabletToolButton(quint32 button, bool pressed, std::chrono::microseconds time);
+    bool sendScrollWheel(double v120, std::chrono::microseconds time);
 
     std::unique_ptr<InputDevice> m_inputDevice;
-    std::array<QHash<Trigger, std::variant<QKeySequence, MouseButton, TabletToolButton, DisabledButton>>, LastType> m_actions;
+    std::array<QHash<Trigger, std::variant<SingleKeybind, AxisKeybind, MouseButton, TabletToolButton, DisabledButton, ScrollWheel>>, LastType> m_actions;
     KConfigWatcher::Ptr m_configWatcher;
     QPointer<KWin::InputDeviceTabletTool> m_tabletTool;
     QPointF m_cursorPos, m_tabletCursorPos;
+    int m_initialRingPosition = -1;
 };
