@@ -11,8 +11,6 @@
 #include "effect/effect.h"
 #include "effect/timeline.h"
 
-#include <xcb/xcb.h>
-
 namespace KWin
 {
 
@@ -22,7 +20,6 @@ class KscreenEffect : public Effect
 
 public:
     KscreenEffect();
-    ~KscreenEffect() override;
 
     void prePaintScreen(ScreenPrePaintData &data, std::chrono::milliseconds presentTime) override;
     void postPaintScreen() override;
@@ -37,9 +34,6 @@ public:
         return 99;
     }
 
-private Q_SLOTS:
-    void propertyNotify(KWin::EffectWindow *window, long atom);
-
 private:
     enum FadeOutState {
         StateNormal,
@@ -59,10 +53,8 @@ private:
     void addScreen(Output *screen);
     bool isScreenActive(Output *screen) const;
 
-    QHash<Output *, ScreenState> m_waylandStates;
-    ScreenState m_xcbState;
+    QHash<Output *, ScreenState> m_states;
     Output *m_currentScreen = nullptr;
-    xcb_atom_t m_atom;
 };
 
 } // namespace KWin
