@@ -14,6 +14,8 @@
 // KConfigSkeleton
 #include "magiclampconfig.h"
 
+using namespace std::chrono_literals;
+
 namespace KWin
 {
 
@@ -43,9 +45,9 @@ void MagicLampEffect::reconfigure(ReconfigureFlags)
 
     // TODO: Rename animationDuration to duration so we can use
     // animationTime<MagicLampConfig>(250).
-    const int d = MagicLampConfig::animationDuration() != 0
-        ? MagicLampConfig::animationDuration()
-        : 250;
+    const std::chrono::milliseconds d = MagicLampConfig::animationDuration() != 0
+        ? std::chrono::milliseconds(MagicLampConfig::animationDuration())
+        : 250ms;
     m_duration = std::chrono::milliseconds(static_cast<int>(animationTime(d)));
 }
 
@@ -144,7 +146,7 @@ void MagicLampEffect::apply(EffectWindow *w, int mask, WindowPaintData &data, Wi
                 }
             }
             if (panel) {
-                // Assumption: width of horizonal panel is greater than its height and vice versa
+                // Assumption: width of horizontal panel is greater than its height and vice versa
                 const QRectF windowScreen = effects->clientArea(ScreenArea, w);
 
                 if (panel->width() >= panel->height()) {
@@ -205,7 +207,7 @@ void MagicLampEffect::apply(EffectWindow *w, int mask, WindowPaintData &data, Wi
         quads = quads.makeGrid(40);
         float quadFactor; // defines how fast a quad is vertically moved: y coordinates near to window top are slowed down
                           // it is used as quadFactor^3/windowHeight^3
-                          // quadFactor is the y position of the quad but is changed towards becomming the window height
+                          // quadFactor is the y position of the quad but is changed towards becoming the window height
                           // by that the factor becomes 1 and has no influence any more
         float offset[2] = {0, 0}; // how far has a quad to be moved? Distance between icon and window multiplied by the progress and by the quadFactor
         float p_progress[2] = {0, 0}; // the factor which defines how far the x values have to be changed
