@@ -7,6 +7,7 @@
 
 #include "expolayout.h"
 
+#include <QQmlProperty>
 #include <cmath>
 #include <deque>
 #include <tuple>
@@ -41,6 +42,12 @@ ExpoCell::~ExpoCell()
 void ExpoCell::componentComplete()
 {
     QQuickItem::componentComplete();
+
+    QQmlProperty xProperty(this, "Kirigami.ScenePosition.x", qmlContext(this));
+    xProperty.connectNotifySignal(this, SLOT(updateContentItemGeometry()));
+    QQmlProperty yProperty(this, "Kirigami.ScenePosition.y", qmlContext(this));
+    yProperty.connectNotifySignal(this, SLOT(updateContentItemGeometry()));
+
     updateContentItemGeometry();
 }
 
@@ -473,7 +480,7 @@ static bool isDominated(size_t candidate, size_t alternativeSmall, size_t altern
         return true;
     }
 
-    // We assumed that the weigth function is concave, i.e., for all i <= j < k <= l,
+    // We assumed that the weight function is concave, i.e., for all i <= j < k <= l,
     //     weight(i,k) + weight(j,l) <= weight(i,l) + weight(j,k)
     // This implies the following about leastWeightCandidate:
     // For all i <= j < k <= l
