@@ -4,7 +4,6 @@
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
-#include "main.h"
 #include "screencastmanager.h"
 
 #include <KPluginFactory>
@@ -25,14 +24,7 @@ public:
 
 std::unique_ptr<Plugin> ScreencastManagerFactory::create() const
 {
-    switch (kwinApp()->operationMode()) {
-    case Application::OperationModeX11:
-        return nullptr;
-    case Application::OperationModeWayland:
-        return std::make_unique<ScreencastManager>();
-    default:
-        return nullptr;
-    }
+    return std::make_unique<ScreencastManager>();
 }
 
 #include "main.moc"
