@@ -7,6 +7,7 @@
 #include "scene/scene.h"
 #include "core/output.h"
 #include "core/outputlayer.h"
+#include "core/pixelgrid.h"
 #include "core/renderviewport.h"
 #include "effect/effect.h"
 #include "scene/cursoritem.h"
@@ -244,6 +245,7 @@ QRectF ItemView::viewport() const
 
 QRectF ItemView::calculateViewport(const QRectF &itemRect) const
 {
+    const QRectF snapped = snapToPixels(itemRect, scale());
     const auto recommendedSizes = m_layer ? m_layer->recommendedSizes() : QList<QSize>{};
     if (!recommendedSizes.empty()) {
         const auto bufferSize = scaledRect(itemRect, scale()).size();
@@ -255,10 +257,10 @@ QRectF ItemView::calculateViewport(const QRectF &itemRect) const
         });
         if (it != bigEnough.end()) {
             const auto logicalSize = QSizeF(*it) / scale();
-            return m_item->mapToView(QRectF(itemRect.topLeft(), logicalSize), this);
+            return m_item->mapToView(QRectF(snapped.topLeft(), logicalSize), this);
         }
     }
-    return m_item->mapToView(itemRect, this);
+    return m_item->mapToView(snapped, this);
 }
 
 bool ItemView::isVisible() const
@@ -430,7 +432,8 @@ QRegion ItemTreeView::collectDamage()
 
 void ItemTreeView::paint(const RenderTarget &renderTarget, const QRegion &region)
 {
-    const QRegion globalRegion = region == infiniteRegion() ? infiniteRegion() : region.translated(viewport().topLeft().toPoint());
+    // FIXME damage tracking for this layer still has some bugs, this effectively disables it
+    const QRegion globalRegion = infiniteRegion();
     RenderViewport renderViewport(viewport(), m_output->scale(), renderTarget);
     auto renderer = m_item->scene()->renderer();
     renderer->beginFrame(renderTarget, renderViewport);
