@@ -121,29 +121,11 @@ public:
      */
     virtual QString desktopName(Window *client) const = 0;
 
-    /**
-     * De-/Elevate a client using the compositor (if enabled)
-     */
-    virtual void elevateClient(Window *c, QWindow *tabbox, bool elevate) const = 0;
-
     /**
      * Raise a client (w/o activating it)
      */
     virtual void raiseClient(Window *c) const = 0;
 
-    /**
-     * @param c The client to be restacked
-     * @param under The client the other one will be placed below
-     */
-    virtual void restack(Window *c, Window *under) = 0;
-
-    /**
-     * Toggle between ShadeHover and ShadeNormal - not shaded windows are unaffected
-     * @param c The client to be shaded
-     * @param b Whether to un- or shade
-     */
-    virtual void shadeClient(Window *c, bool b) const = 0;
-
     virtual void highlightWindows(Window *window = nullptr, QWindow *controller = nullptr) = 0;
 
     /**
