@@ -15,6 +15,8 @@
 #include "input_event_spy.h"
 #include "osd.h"
 #include "pointer_input.h"
+#include "wayland/display.h"
+#include "wayland/seat.h"
 #include "wayland/tablet_v2.h"
 #include "wayland_server.h"
 #include "window.h"
@@ -235,8 +237,8 @@ void TabletInputRedirection::tabletToolAxisEvent(const QPointF &pos, qreal press
         .tool = tool,
     };
 
-    input()->processSpies(std::bind(&InputEventSpy::tabletToolAxisEvent, std::placeholders::_1, &ev));
-    input()->processFilters(std::bind(&InputEventFilter::tabletToolAxisEvent, std::placeholders::_1, &ev));
+    input()->processSpies(&InputEventSpy::tabletToolAxisEvent, &ev);
+    input()->processFilters(&InputEventFilter::tabletToolAxisEvent, &ev);
     input()->setLastInputHandler(this);
 }
 
@@ -275,8 +277,8 @@ void TabletInputRedirection::tabletToolAxisEventRelative(const QPointF &delta,
         .tool = tool,
     };
 
-    input()->processSpies(std::bind(&InputEventSpy::tabletToolAxisEvent, std::placeholders::_1, &ev));
-    input()->processFilters(std::bind(&InputEventFilter::tabletToolAxisEvent, std::placeholders::_1, &ev));
+    input()->processSpies(&InputEventSpy::tabletToolAxisEvent, &ev);
+    input()->processFilters(&InputEventFilter::tabletToolAxisEvent, &ev);
     input()->setLastInputHandler(this);
 }
 
@@ -312,8 +314,8 @@ void TabletInputRedirection::tabletToolProximityEvent(const QPointF &pos, qreal
         .tool = tool,
     };
 
-    input()->processSpies(std::bind(&InputEventSpy::tabletToolProximityEvent, std::placeholders::_1, &ev));
-    input()->processFilters(std::bind(&InputEventFilter::tabletToolProximityEvent, std::placeholders::_1, &ev));
+    input()->processSpies(&InputEventSpy::tabletToolProximityEvent, &ev);
+    input()->processFilters(&InputEventFilter::tabletToolProximityEvent, &ev);
     input()->setLastInputHandler(this);
 }
 
@@ -360,9 +362,15 @@ void TabletInputRedirection::tabletToolTipEvent(const QPointF &pos, qreal pressu
         .tool = tool,
     };
 
-    input()->processSpies(std::bind(&InputEventSpy::tabletToolTipEvent, std::placeholders::_1, &ev));
-    input()->processFilters(std::bind(&InputEventFilter::tabletToolTipEvent, std::placeholders::_1, &ev));
+    input()->processSpies(&InputEventSpy::tabletToolTipEvent, &ev);
+    input()->processFilters(&InputEventFilter::tabletToolTipEvent, &ev);
     input()->setLastInputHandler(this);
+    if (tipDown) {
+        input()->setLastInteractionSerial(waylandServer()->seat()->display()->serial());
+        if (auto f = focus()) {
+            f->setLastUsageSerial(waylandServer()->seat()->display()->serial());
+        }
+    }
 }
 
 void KWin::TabletInputRedirection::tabletToolButtonEvent(uint button, bool isPressed, InputDeviceTabletTool *tool, std::chrono::microseconds time, InputDevice *device)
@@ -379,9 +387,15 @@ void KWin::TabletInputRedirection::tabletToolButtonEvent(uint button, bool isPre
 
     m_buttonDown = isPressed;
 
-    input()->processSpies(std::bind(&InputEventSpy::tabletToolButtonEvent, std::placeholders::_1, &event));
-    input()->processFilters(std::bind(&InputEventFilter::tabletToolButtonEvent, std::placeholders::_1, &event));
+    input()->processSpies(&InputEventSpy::tabletToolButtonEvent, &event);
+    input()->processFilters(&InputEventFilter::tabletToolButtonEvent, &event);
     input()->setLastInputHandler(this);
+    if (isPressed) {
+        input()->setLastInteractionSerial(waylandServer()->seat()->display()->serial());
+        if (auto f = focus()) {
+            f->setLastUsageSerial(waylandServer()->seat()->display()->serial());
+        }
+    }
 }
 
 void KWin::TabletInputRedirection::tabletPadButtonEvent(uint button, bool isPressed, quint32 group, quint32 mode, bool isModeSwitch, std::chrono::microseconds time, InputDevice *device)
@@ -395,12 +409,18 @@ void KWin::TabletInputRedirection::tabletPadButtonEvent(uint button, bool isPres
         .isModeSwitch = isModeSwitch,
         .time = time,
     };
-    input()->processSpies(std::bind(&InputEventSpy::tabletPadButtonEvent, std::placeholders::_1, &event));
-    input()->processFilters(std::bind(&InputEventFilter::tabletPadButtonEvent, std::placeholders::_1, &event));
+    input()->processSpies(&InputEventSpy::tabletPadButtonEvent, &event);
+    input()->processFilters(&InputEventFilter::tabletPadButtonEvent, &event);
     input()->setLastInputHandler(this);
+    if (isPressed) {
+        input()->setLastInteractionSerial(waylandServer()->seat()->display()->serial());
+        if (auto f = focus()) {
+            f->setLastUsageSerial(waylandServer()->seat()->display()->serial());
+        }
+    }
 }
 
-void KWin::TabletInputRedirection::tabletPadStripEvent(int number, int position, bool isFinger, quint32 group, std::chrono::microseconds time, InputDevice *device)
+void KWin::TabletInputRedirection::tabletPadStripEvent(int number, int position, bool isFinger, quint32 group, quint32 mode, std::chrono::microseconds time, InputDevice *device)
 {
     TabletPadStripEvent event{
         .device = device,
@@ -408,15 +428,16 @@ void KWin::TabletInputRedirection::tabletPadStripEvent(int number, int position,
         .position = position,
         .isFinger = isFinger,
         .group = group,
+        .mode = mode,
         .time = time,
     };
 
-    input()->processSpies(std::bind(&InputEventSpy::tabletPadStripEvent, std::placeholders::_1, &event));
-    input()->processFilters(std::bind(&InputEventFilter::tabletPadStripEvent, std::placeholders::_1, &event));
+    input()->processSpies(&InputEventSpy::tabletPadStripEvent, &event);
+    input()->processFilters(&InputEventFilter::tabletPadStripEvent, &event);
     input()->setLastInputHandler(this);
 }
 
-void KWin::TabletInputRedirection::tabletPadRingEvent(int number, int position, bool isFinger, quint32 group, std::chrono::microseconds time, InputDevice *device)
+void KWin::TabletInputRedirection::tabletPadRingEvent(int number, int position, bool isFinger, quint32 group, quint32 mode, std::chrono::microseconds time, InputDevice *device)
 {
     TabletPadRingEvent event{
         .device = device,
@@ -424,11 +445,12 @@ void KWin::TabletInputRedirection::tabletPadRingEvent(int number, int position,
         .position = position,
         .isFinger = isFinger,
         .group = group,
+        .mode = mode,
         .time = time,
     };
 
-    input()->processSpies(std::bind(&InputEventSpy::tabletPadRingEvent, std::placeholders::_1, &event));
-    input()->processFilters(std::bind(&InputEventFilter::tabletPadRingEvent, std::placeholders::_1, &event));
+    input()->processSpies(&InputEventSpy::tabletPadRingEvent, &event);
+    input()->processFilters(&InputEventFilter::tabletPadRingEvent, &event);
     input()->setLastInputHandler(this);
 }
 
@@ -443,8 +465,8 @@ void KWin::TabletInputRedirection::tabletPadDialEvent(int number, double delta,
         .time = time,
     };
 
-    input()->processSpies(std::bind(&InputEventSpy::tabletPadDialEvent, std::placeholders::_1, &event));
-    input()->processFilters(std::bind(&InputEventFilter::tabletPadDialEvent, std::placeholders::_1, &event));
+    input()->processSpies(&InputEventSpy::tabletPadDialEvent, &event);
+    input()->processFilters(&InputEventFilter::tabletPadDialEvent, &event);
     input()->setLastInputHandler(this);
 }
 
