@@ -25,7 +25,6 @@ public:
 
     PrimarySelectionDeviceV1Interface *q;
     SeatInterface *seat;
-    QPointer<PrimarySelectionSourceV1Interface> selection;
 
 private:
     void setSelection(PrimarySelectionSourceV1Interface *dataSource);
@@ -54,14 +53,7 @@ void PrimarySelectionDeviceV1InterfacePrivate::zwp_primary_selection_device_v1_s
         Q_ASSERT(dataSource);
     }
 
-    if (selection == dataSource) {
-        return;
-    }
-    if (selection) {
-        selection->cancel();
-    }
-    selection = dataSource;
-    Q_EMIT q->selectionChanged(selection, serial);
+    Q_EMIT q->selectionChanged(dataSource, serial);
 }
 
 void PrimarySelectionDeviceV1InterfacePrivate::zwp_primary_selection_device_v1_destroy(QtWaylandServer::zwp_primary_selection_device_v1::Resource *resource)
@@ -109,15 +101,11 @@ SeatInterface *PrimarySelectionDeviceV1Interface::seat() const
     return d->seat;
 }
 
-PrimarySelectionSourceV1Interface *PrimarySelectionDeviceV1Interface::selection() const
-{
-    return d->selection;
-}
-
-void PrimarySelectionDeviceV1Interface::sendSelection(AbstractDataSource *other)
+PrimarySelectionOfferV1Interface *PrimarySelectionDeviceV1Interface::sendSelection(AbstractDataSource *other)
 {
     PrimarySelectionOfferV1Interface *offer = d->createDataOffer(other);
     d->send_selection(offer ? offer->resource() : nullptr);
+    return offer;
 }
 
 wl_client *PrimarySelectionDeviceV1Interface::client() const
