@@ -9,6 +9,7 @@
 
 #include "xdgactivationv1.h"
 #include "effect/effecthandler.h"
+#include "input.h"
 #include "utils/common.h"
 #include "wayland/clientconnection.h"
 #include "wayland/display.h"
@@ -18,6 +19,7 @@
 #include "wayland_server.h"
 #include "window.h"
 #include "workspace.h"
+
 #include <KDesktopFile>
 
 namespace KWin
@@ -33,32 +35,16 @@ static bool isPrivilegedInWindowManagement(const ClientConnection *client)
 XdgActivationV1Integration::XdgActivationV1Integration(XdgActivationV1Interface *activation, QObject *parent)
     : QObject(parent)
 {
-    Workspace *ws = Workspace::self();
-    connect(ws, &Workspace::windowActivated, this, [this](Window *window) {
-        if (!m_currentActivationToken || !window || window->property("token").toString() == m_currentActivationToken->token) {
+    connect(Workspace::self(), &Workspace::windowActivated, this, [this](Window *window) {
+        if (!m_activation || !window || m_lastTokenAppId != window->desktopFileName()) {
             return;
         }
-
-        // We check that it's not the app that we are trying to activate
-        if (window->desktopFileName() != m_currentActivationToken->applicationId) {
-            // But also that the new one has been requested after the token was requested
-            if (window->lastUsageSerial() < m_currentActivationToken->serial) {
-                return;
-            }
-        }
-
-        clear();
+        clearFeedback();
     });
-    activation->setActivationTokenCreator([this](ClientConnection *client, SurfaceInterface *surface, uint serial, SeatInterface *seat, const QString &appId) -> QString {
-        Workspace *ws = Workspace::self();
-        Q_ASSERT(client); // Should always be available as it's coming straight from the wayland implementation
-        const bool isPrivileged = isPrivilegedInWindowManagement(client);
-        if (!isPrivileged && ws->activeWindow() && ws->activeWindow()->surface() != surface) {
-            qCDebug(KWIN_CORE) << "Cannot grant a token to" << client;
-            return QStringLiteral("not-granted-666");
-        }
 
-        return requestToken(isPrivileged, surface, serial, seat, appId);
+    activation->setActivationTokenCreator([this](ClientConnection *client, SurfaceInterface *surface, uint serial, SeatInterface *seat, const QString &appId) -> QString {
+        Q_ASSERT(client); // Should always be available as it's coming straight from the wayland implementation
+        return requestToken(isPrivilegedInWindowManagement(client), surface, serial, seat, appId);
     });
 
     connect(activation, &XdgActivationV1Interface::activateRequested, this, &XdgActivationV1Integration::activateSurface);
@@ -66,12 +52,19 @@ XdgActivationV1Integration::XdgActivationV1Integration(XdgActivationV1Interface
 
 QString XdgActivationV1Integration::requestToken(bool isPrivileged, SurfaceInterface *surface, uint serial, SeatInterface *seat, const QString &appId)
 {
+    auto window = waylandServer()->findWindow(surface);
+    if (!isPrivileged) {
+        const bool allowed = !workspace()->activeWindow()
+            || workspace()->activeWindow()->surface() == surface
+            || (input()->lastInteractionSerial() <= serial && waylandServer()->display()->serial() >= serial);
+        if (!allowed) {
+            qCDebug(KWIN_CORE) << "Cannot grant a token to" << window;
+            return QStringLiteral("not-granted-666");
+        }
+    }
     static int i = 0;
     const auto newToken = QStringLiteral("kwin-%1").arg(++i);
 
-    if (m_currentActivationToken) {
-        clear();
-    }
     bool showNotify = false;
     QIcon icon = QIcon::fromTheme(QStringLiteral("system-run"));
     if (const QString desktopFilePath = Window::findDesktopFile(appId); !desktopFilePath.isEmpty()) {
@@ -83,13 +76,21 @@ QString XdgActivationV1Integration::requestToken(bool isPrivileged, SurfaceInter
         }
         icon = QIcon::fromTheme(df.readIcon(), icon);
     }
-    std::unique_ptr<PlasmaWindowActivationInterface> activation;
     if (showNotify) {
-        activation = waylandServer()->plasmaActivationFeedback()->createActivation(appId);
+        m_lastToken = newToken;
+        m_lastTokenAppId = appId;
+        m_activation = waylandServer()->plasmaActivationFeedback()->createActivation(appId);
     }
-    m_currentActivationToken = std::make_unique<ActivationToken>(ActivationToken{newToken, isPrivileged, surface, serial, seat, appId, showNotify, std::move(activation)});
+    if (isPrivileged) {
+        // plasmashell doesn't have a valid serial unless it has keyboard focus,
+        // this makes it so that shortcuts like Meta+1 work
+        // TODO pass seat + serial to plasmashell through kglobalaccel to fix this
+        // more properly
+        serial = input()->lastInteractionSerial();
+    }
+    workspace()->setActivationToken(newToken, serial, appId);
     if (showNotify) {
-        Q_EMIT effects->startupAdded(m_currentActivationToken->token, icon);
+        Q_EMIT effects->startupAdded(newToken, icon);
     }
     return newToken;
 }
@@ -103,31 +104,24 @@ void XdgActivationV1Integration::activateSurface(SurfaceInterface *surface, cons
         return;
     }
 
-    if (!m_currentActivationToken || m_currentActivationToken->token != token) {
-        qCDebug(KWIN_CORE) << "Refusing to activate " << window << " (provided token: " << token << ", current token:" << (m_currentActivationToken ? m_currentActivationToken->token : QStringLiteral("null")) << ")";
+    if (!ws->mayActivate(window, token)) {
         window->demandAttention();
         return;
     }
-
-    auto ownerWindow = waylandServer()->findWindow(m_currentActivationToken->surface);
-    qCDebug(KWIN_CORE) << "activating" << window << surface << "on behalf of" << m_currentActivationToken->surface << "into" << ownerWindow;
-    if (!ws->activeWindow() || ws->activeWindow() == ownerWindow || ws->activeWindow()->lastUsageSerial() < m_currentActivationToken->serial || m_currentActivationToken->isPrivileged) {
+    if (window->readyForPainting()) {
         ws->activateWindow(window);
     } else {
-        qCWarning(KWIN_CORE) << "Activation requested while owner isn't active" << (ownerWindow ? ownerWindow->desktopFileName() : "null")
-                             << m_currentActivationToken->applicationId;
-        window->demandAttention();
-        clear();
+        window->setActivationToken(token);
     }
+    clearFeedback();
 }
 
-void XdgActivationV1Integration::clear()
+void XdgActivationV1Integration::clearFeedback()
 {
-    Q_ASSERT(m_currentActivationToken);
-    if (m_currentActivationToken->showNotify) {
-        Q_EMIT effects->startupRemoved(m_currentActivationToken->token);
+    if (m_activation) {
+        Q_EMIT effects->startupRemoved(m_lastToken);
+        m_activation.reset();
     }
-    m_currentActivationToken.reset();
 }
 
 }
